import sys
import os
import pickle
from PyQt5.QtWidgets import*
from PyQt5.QtCore import *
from PyQt5.Qt import QStandardItemModel, QStandardItem
from PyQt5.QtGui import *
import GraphKey.app.modules.data_export.google_drive_interface.google_drive_functions as gd
from GraphKey.app.modules import path_globals
from pathlib import Path

class GoogleDriveAccountSettings(QDialog):
    """
    A QWidget class defining the Google Drive Account Settings Screen. This screen shows the user the currently selected Google Drive account and gives them the option to select a new account, add an account, or delete an account.
    """

    def __init__(self, app, graph_page):
        """Initializes the Google Drive Account Settings Screen.
           :param: app - Variable pointing to the parent application, used for processing events.
           :param: graph_page - Variable linking to the graph_creation_page instance.
        """
        super().__init__()
        self.widget_app = app
        self.graph_creation_page = graph_page
        exists = os.path.exists(Path(path_globals.settings_folder_path / "google_drive_configs.p"))
        if (not exists):
            configs = {'saved_accounts': [], 'current_account': {'name': 'None', 'account': None}}
            pickle.dump(configs, open(Path(path_globals.settings_folder_path / "google_drive_configs.p"), "w+b"))
        configs = pickle.load(open(Path(path_globals.settings_folder_path / "google_drive_configs.p"), "rb"))
        self.current_account = configs["current_account"]
        self.saved_accounts = configs["saved_accounts"]
        self.make_window()

    def make_window(self):
        """Creates the Account Settings window and all of its elements."""
        
        layout = QVBoxLayout()

        ####GOOGLE DRIVE SETTINGS####
        drive_layout = QVBoxLayout()

        #CURRENT ACCOUNT AREA
        current_account_selected = QLabel("Current Account Selected: {}".format(self.current_account["name"]))

        #SAVED ACCOUNT AREA
        radio_button_list = []
        scroll_widget = QWidget()
        scroll_layout = QVBoxLayout()
        different_theme = QLabel("Saved Accounts")
        i = 0
        for t in self.saved_accounts:
            radio_button_list.append(QRadioButton("{}".format(t["name"])))
            scroll_layout.addWidget(radio_button_list[i])
            i = i+1
        scroll_widget.setLayout(scroll_layout)

        scroll_theme = QScrollArea()
        scroll_theme.setWidget(scroll_widget)
        scroll_theme.setWidgetResizable(True)


        #BUTTON AREA
        button_widget = QWidget()
        button_layout = QHBoxLayout()
        apply_changes = QPushButton("Apply Changes")
        apply_changes.clicked.connect(lambda: self.apply_changes())
        add = QPushButton("Add Account")
        add.clicked.connect(lambda: self.add_account())
        select = QPushButton("Select Account")
        select.clicked.connect(lambda: self.select_account(radio_button_list))
        cancel = QPushButton("Cancel")
        cancel.clicked.connect(lambda: self.cancel())
        delete = QPushButton("Delete Account")
        delete.clicked.connect(lambda: self.delete_account(radio_button_list))
        
        button_layout.addWidget(select)
        button_layout.addWidget(add)
        button_layout.addWidget(delete)
        button_layout.addWidget(apply_changes)
        button_layout.addWidget(cancel)
        button_widget.setLayout(button_layout)

        #WHOLE WIDGET
        drive_layout.addWidget(current_account_selected)
        drive_layout.addWidget(different_theme)
        drive_layout.addWidget(scroll_theme)
        drive_layout.addWidget(button_widget)
        drive_widget = QWidget()
        drive_widget.setLayout(drive_layout)

        layout.addWidget(drive_widget)
        self.setLayout(layout)
        self.setWindowTitle("Switch Google Drive Accounts")

    def apply_changes(self):
        """This functions saves the currently selected Google Drive account and all saved Google Drive accounts to a pickle file."""
        configs = {"saved_accounts": self.saved_accounts, "current_account": self.current_account}
        pickle.dump(configs, open(Path(path_globals.settings_folder_path / "google_drive_configs.p"), "wb"))
        self.close()
        return

    def select_account(self, buttons):
        """If the user selects a new account, this function will switch the currently selected account.
           :param: buttons - The buttons connected to each saved Google Drive account, used for getting the selected account.
        """
        selected = self.get_selected(buttons)
        if (selected is None):
            msg = QMessageBox()
            msg.setText("No account was selected, failed to select account.")
            msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
            msg.setWindowTitle("Failed to Select Account")
            msg.exec()
            return
        self.current_account = self.saved_accounts[selected]
        self.apply_changes()
        setting = GoogleDriveAccountSettings(self.widget_app, self.graph_creation_page) 
        setting.exec()

    def add_account(self):
        """If the user chooses to add an account, this function will take the user through the Google Drive authentication process and save the new account."""
        drive = gd.init_drive()
        if (drive == "AuthError") or (drive == "TimeoutError"):
            print(drive)
            msg = QMessageBox()
            msg.setText("An error occurred while authenticating your account. Please try again.")
            msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
            msg.setWindowTitle("Error")
            msg.exec()
            self.close()
            return
        names = [account_name["name"] for account_name in self.saved_accounts]
        name, ok = QInputDialog.getText(self, "Account Name", "Enter an Account Name:")
        if ok:
            while (name in names or name == "None"):
                name, ok = QInputDialog.getText(self, 'Account Name', 'Account name already exists. Enter a new Account Name:')
                if ok:
                    continue
                else:
                    break

            self.saved_accounts.append({"name":name, "account":drive})
            self.apply_changes()
            setting = GoogleDriveAccountSettings(self.widget_app, self.graph_creation_page) 
            setting.exec()

    def cancel(self):
        """If the user chooses to cancel, this function will close the window."""
        self.close()

    def delete_account(self, buttons):
        """If the user chooses to delete an account, this function will delete the currently selected account.
           :param: buttons - The buttons connected to each saved Google Drive account, used for getting the selected account.
        """
        selected = self.get_selected(buttons)
        if (selected is None):
            msg = QMessageBox()
            msg.setText("No account was selected, failed to delete acconut.")
            msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
            msg.setWindowTitle("Failed to Delete Account")
            msg.exec()
            return
        if (self.saved_accounts[selected]["name"] == self.current_account["name"]):
            self.current_account = {"name":"None", "account":None}
        self.saved_accounts.pop(selected)
        self.apply_changes()
        setting = GoogleDriveAccountSettings(self.widget_app, self.graph_creation_page) 
        setting.exec()
    
    def get_selected(self, buttons):
        """This functions gets the currently selected account.
           :param: buttons - The buttons connected to each saved Google Drive account, used for getting the selected account.
           :return: Returns the index of the selected account.
        """
        i = 0
        for button in buttons:
            if button.isChecked():
                return i
            i = i+1

