import sys
import os
import pickle
import PIL as pil
from PIL import Image
from PyQt5.QtWidgets import*
from PyQt5.QtCore import *
from PyQt5.Qt import QStandardItemModel, QStandardItem
from PyQt5.QtGui import *
import GraphKey.app.modules.data_export.google_drive_interface.google_drive_functions as gd
from GraphKey.app.modules.graphing.generate_graph_pickle import *
from GraphKey.app.modules.gui.graph_creation_window import *
from GraphKey.app.modules import path_globals

class StandardItem(QStandardItem):
    """A QStandardItem class that represent a folder or file object in the Google Drive account."""

    file_id = 0
    folder = False

    def __init__(self, txt='', font_size=12, set_bold=False, color=QColor(0, 0, 0)):
        """This function initializes the item.
           :param: txt - The text of the item.
           :param: font_size - The font_size of the text of the item.
           :param: set_bold - Enables if the text of the item should be bold.
           :param: color - The color of the text of the item.
        """
        super().__init__()

        fnt = QFont('Times New Roman', font_size)
        fnt.setBold(set_bold)

        self.setEditable(False)
        self.setForeground(color)
        self.setFont(fnt)
        self.setText(txt)

    def set_file_id(self, f_id):
        """This function gives the item a file id.
           :param: f_id - The file id to assign.
        """
        self.file_id = f_id

    def get_file_id(self):
        """This function returns the file id of an item.
           :return: File id.
        """
        return self.file_id

    def set_folder(self, flag):
        """Thus function tells the item if it is a folder.
           :param: flag - Boolean defining if the item is a folder.
        """
        self.folder = flag

    def get_folder(self):
        """This functions returns if the item is a folder.
           :return: Boolean defining if the item is a folder.
        """
        return self.folder

class _SortModel(QSortFilterProxyModel):
    """A QSortFilterProxyModel class helping to sort the Google Drive contents."""
    
    def __init__(self, model):
        """Sorting proxy model that always places folders on top."""
        super().__init__()
        self.setSourceModel(model)

    def lessThan(self, left, right):
        """Perform sorting comparison. Since we know the sort order, we can ensure that folders always come first.
           :return: Returns the order to put the items in.
        """
        left_is_folder = left.data(Qt.UserRole)
        left_data = left.data(Qt.DisplayRole)
        right_is_folder = right.data(Qt.UserRole)
        right_data = right.data(Qt.DisplayRole)
        sort_order = self.sortOrder()

        if left_is_folder and not right_is_folder:
            result = sort_order == Qt.AscendingOrder
        elif not left_is_folder and right_is_folder:
            result = sort_order != Qt.AscendingOrder
        else:
            result = left_data > right_data
        return result

class FileTree(QWidget):
    """
    A QWidget class defining the Google Drive Select Folder Screen. This screen shows the user the folders they can select from to export their files to on their Google Drive account.
    """

    def __init__(self, app, drive, list_of_file_paths, graph_page):
        """Initializes the Google Drive Select Folder Screen.
           :param: app - Variable pointing to the parent application, used for processing events.
           :param: drive - The Google Drive account to upload to.
           :param: list_of_file_paths - List of files user can choose to export from.
           :param: graph_page - Variable linking to the graph_creation_page instance.
        """
        super().__init__()
        self.widget_app = app
        self.graph_creation_page = graph_page
        self.make_window(drive, list_of_file_paths)

    def make_window(self, drive, list_of_file_paths):
        """Creates the Select Folder window and all of its elements."""
        msg = QMessageBox()
        msg.setWindowTitle("Retrieving files...")
        msg.setText("Please wait while your Google Drive contents are retrieved. Do not close this window.")
        msg.setModal(True)
        msg.show()
        self.widget_app.processEvents()
        self.setWindowTitle('Select Destination Folder')
        self.resize(500, 700)
        
        layout = QVBoxLayout()
        button_layout = QHBoxLayout()

        treeView = QTreeView()
        treeView.setHeaderHidden(True)
        
        treeModel = QStandardItemModel()
        rootNode = treeModel.invisibleRootItem()
 
        self.recursive_file_getter(12, 0, drive, rootNode)
        sorted_model = _SortModel(treeModel)    
        treeView.setModel(sorted_model)
        treeView.setSortingEnabled(True)
        treeView.collapseAll()
        treeView.setSelectionMode(QTreeView.SingleSelection)

        info = QLabel()
        info.setText("Note: \t File(s) will be saved to the selected folder - if no folder is selected, the file will be saved to the root directory in Google Drive. \n\t Folders will be created in the selected folder - if no folder is selected, the folder will be created in the root directory in Google Drive. \n\t To de-select a selected folder, hold CTRL and click.\n")

        create_folder_button = QPushButton("Create New Folder")
        create_folder_button.setCheckable(False)
        create_folder_button.clicked.connect(lambda: self.create_folder(drive, treeView, sorted_model, treeModel, list_of_file_paths))

        save_button = QPushButton("Save File(s)")
        save_button.setCheckable(False)
        save_button.clicked.connect(lambda: self.save_file(list_of_file_paths, treeView, sorted_model, treeModel, drive))

        cancel_button = QPushButton("Cancel")
        cancel_button.clicked.connect(lambda: self.cancel())

        button_layout.addWidget(create_folder_button)
        button_layout.addWidget(save_button)
        button_layout.addWidget(cancel_button)
        button_widget = QWidget()
        button_widget.setLayout(button_layout)

        layout.addWidget(info)
        layout.addWidget(treeView)
        layout.addWidget(button_widget)
        self.setLayout(layout)
        msg.close()

    def recursive_file_getter(self, font_size, current_folder_id, drive, parent):
        """This function retrieves the contents of a Google Drive account in a recursive fashion.
           :param: font_size - The current font size of this level of recursion.
           :param: current_folder_if - The id of the folder we're currently looking in.
           :param: drive - The link to the Google Drive account.
           :param: parent - The parent folder.
        """
        fileList = gd.list_folders_files(drive, current_folder_id)
        for file in fileList:
            item = StandardItem(file['title'], font_size, set_bold=False)
            item.set_file_id(file['id'])
            item.set_folder(False)
            item.setData(False, Qt.UserRole)
            if(file['mimeType'] == "application/vnd.google-apps.folder"):
                item.setData(True, Qt.UserRole)
                item.set_folder(True)
                self.recursive_file_getter(font_size, file['id'], drive, item)
            parent.appendRow(item)

    def getSelectedItem(self, treeView, sorted_model, treeModel):
        """This function gets the currently selected item.
           :param: treeView - The file tree object, used to get the selected item.
           :param: sorted_model - The model mapping to the proper index of an item.
           :param: treeModel - The model containing the item nodes.
           :return: The selected item node.
        """
        list_index = treeView.selectedIndexes()
        if (len(list_index) < 1):
                return 0
        source_index = sorted_model.mapToSource(list_index[0])
        return treeModel.itemFromIndex(source_index)

    def save_file(self, list_of_file_paths, treeView, sorted_model, treeModel, drive):
        """Once the user chooses to export, this function will uploads the files to the Google Drive account.
           :param: list_of_file_paths - The paths of the files to upload.
           :param: treeView - The file tree object, used to get the selected item.
           :param: sorted_model - The model mapping to the proper index of an item.
           :param: treeModel - The model containing the item nodes.
           :param: drive - The link to the Google Drive account.
        """
        item = self.getSelectedItem(treeView, sorted_model, treeModel)
        if (item == 0):
            file_id = 0
        else:
            if (item.get_folder() == False):
                msg = QMessageBox()
                msg.setWindowTitle("Not a folder")
                msg.setText("Please select a folder to save your file(s) in.")
                msg.setModal(True)
                msg.exec_()
                return
            else:
                file_id = item.get_file_id()
        
        msg = QMessageBox()
        msg.setWindowTitle("Saving")
        msg.setText("Please wait while your files are uploaded to Google Drive. Do not close this window.")
        msg.setModal(True)
        msg.show()
        self.widget_app.processEvents()
        for path in list_of_file_paths:
            figure, figure_dict = self.graph_creation_page.single_graph_generation_for_export(path)
            if figure is None:
                return
            figure.write_image(file = path_globals.tmp_folder_path_str + "temp_image.png", format = 'png', scale = figure_dict["scale_factor"], width = figure_dict["width"], height = figure_dict["height"], validate = True)
            fig_image = pil.Image.open(path_globals.tmp_folder_path_str + "temp_image.png")
            fig_image.save(path_globals.tmp_folder_path_str + "temp_image.png", dpi=(figure_dict["dpi"],figure_dict["dpi"]))
            gd.upload_file(drive, path_globals.tmp_folder_path_str + "temp_image.png", file_id, os.path.splitext(os.path.basename(path))[0])
        msg.close()
        self.close()
    
    def create_folder(self, drive, treeView, sorted_model, treeModel, list_of_file_paths):
        """If the user chooses to create a folder, this function will create a folder in the Google Drive account.
           :param: drive - The link to the Google Drive account.
           :param: treeView - The file tree object, used to get the selected item.
           :param: sorted_model - The model mapping to the proper index of an item.
           :param: treeModel - The model containing the item nodes.
           :param: list_of_file_paths - The paths of the files to upload.
        """
        item = self.getSelectedItem(treeView, sorted_model, treeModel)
        if (item == 0):
            file_id = 0
        else:
            if (item.get_folder() == False):
                msg = QMessageBox()
                msg.setWindowTitle("Not a folder")
                msg.setText("Please select a folder to create a new folder in it.")
                msg.setModal(True)
                msg.exec_()
                return
            else:
                file_id = item.get_file_id()
        name, ok = QInputDialog.getText(self, 'Input Dialog', 'Enter New Folder Name:')
        if ok:
            foldername = name
            gd.create_folder(drive, foldername, file_id)
            self.refresh(drive, list_of_file_paths)
    
    def refresh(self, drive, list_of_file_paths):
        """This function refreshes the window, used to update the contents of the Google Drive account.
           :param: drive - The link to the Google Drive account.
           :param: list_of_file_paths - The paths of the files to upload.
        """
        msg = QMessageBox()
        msg.setWindowTitle("Refreshing")
        msg.setText("Please wait while the window refreshes. Do not close this window.")
        msg.setModal(True)
        msg.show()
        self.widget_app.processEvents()
        tree = FileTree(self.widget_app, drive, list_of_file_paths, self.graph_creation_page)
        msg.close()
        self.close()
        tree.show()

    def cancel(self):
        """If the user chooses to cancel, this function closes the window."""
        self.close()
