import pandas as pd

def filter_df(df, column_to_filter, groups_to_remove):
    """Retuns a filtered copy of dataframe "df." Rows that have a value in the "groups_to_remove array" in the column "column_to_filter" are removed.
       :param: df - The data frame that will have a filtered copy of itself returned.
       :param: column_to_filter - The name of the column in the data frame to be filtered.
       :param: A list of the groups to remove from column column_to_filter.
       :return: A data frame with values filtered out as specified by the parameters.
    """
    #create copy of unfiltered dataframe
    filtered_df = df.copy()
    #remove rows in dataframe that have value "group" in column "column_to_filter"
    for group in groups_to_remove:
        filtered_df = filtered_df.loc[(filtered_df[column_to_filter]) != group]
    #return filtered dataframe
    return filtered_df

def filter_using_filter_dict(df, filter_dict:dict):
    """Returns a filtered copy of dataframe "df." Each key in "filter_dict" is a column to filter and the value for that key is an array containing the values to filter from that column
       :param: df - The data frame that will have a filtered version of itself returned.
       :param: filter_dict - A dictionary with column names as keys and arrays containing the values to be filtered from a column as values.
       :return: A data frame with values filtered out as specified by the filter_dict.
    """
    filtered_df = df.copy()
    for key in filter_dict:
        vals_to_filter = filter_dict[key]
        filtered_df = filter_df(filtered_df, key, vals_to_filter)
    return filtered_df

def get_non_group_columns_list(df:pd.DataFrame):
    """Returns a list of columns that do not likely contain group data.
       :param: df - The data frame to process.
       :return: List containing columns that do not likely contain group data.
    """
    col_headers = df.columns.tolist()
    non_group_columns_list = []
    for col in col_headers:
        if(df[col].dtype != "object"):
            non_group_columns_list.append(str(col))
    return non_group_columns_list


def get_group_columns(df:pd.DataFrame):
    """Returns a list of columns that likely contain group data
       :param: df - The data frame to process.
       :return: List containing columns that likely contain group data.
    """
    col_headers = df.columns.tolist()
    group_col_list = []
    for col in col_headers:
        #if this column has data type object, it likely contains strings defining a group
        if(df[col].dtype == "object"):
            group_col_list.append(col)
    return group_col_list
        

def get_group_filtering_dict(df:pd.DataFrame):
    """Returns a dictionary that maps a column name to an array containing it's unique values
       :param: df - The data frame to process.
       :return: A dictionary mapping column names to a list containing all of the unique values in the column.
    """
    #create dictionary to return
    group_filtering_dict = dict()
    #get list of columns that contain group data
    group_col_list = get_group_columns(df)
    #create entries for every column in this list
    for group_col in group_col_list:
        #get unique values in column
        unique_col_values = df[group_col].unique()
        #create an entry for this column in the dictionary
        group_filtering_dict[group_col] = []
        #add the unique values for this column to the array corresponding to it in the dictionary
        for unique_val in unique_col_values:
            group_filtering_dict[group_col].append(unique_val)
    return group_filtering_dict


def create_df_with_new_grouped_column(df:pd.DataFrame, col_header_list, delimiter:str):
    """Returns a new data frame with a combined column. Combined column contains the data from columns in "col_header_list" appended together with "delimiter" in between the appended data
       :param: df - The data frame to process.
       :param: col_header_list - The list of columns to be appended together.
       :param: delimiter - The delimiting string to be used when appending the columns and data together.
       :return: A data frame with a new grouped column made.
    """
    new_df = df.copy()
    new_df[delimiter.join(col_header_list)] = new_df[col_header_list].agg(delimiter.join, axis = 1)
    return new_df

def compare_data_frames(df1:pd.DataFrame, df2:pd.DataFrame):
    """Compares two pandas data frames. Returns true if they are equal. False if they are not equal
       :param: df1 - First data frame to compare.
       :param: df2 - The data frame being compared against df1.
       :return: True if data frames are equal, false if not.
    """    
    return df1.equals(df2)

def write_data_frame_to_pickle(df:pd.DataFrame, file_path:str):
    """Writes dataframe df to a pickle at file path file_path
       :param: df - The data frame to process.
       :param: file_path - The file path to which the data frame will be written to as a pickle.
    """
    df.to_pickle(file_path)

def read_data_frame_from_pickle(file_path:str):
    """Returns a data frame read from a pickle file at path file_path
       :param: file_path - The file path to the pickled data frame.
       :return: The data frame stored in the pickle file.
    """
    return pd.read_pickle(file_path)

def get_sem_df_for_groups_in_column(df:pd.DataFrame, group_list):    
    """Returns grouped data frame for the groups in the column.
       :param: df - The data frame to process.
       :param: group_list - The list of groups columns that will have the standard error of mean calculated for
       :return: A dataframe containing standard error of mean info.
    """
    return df.groupby(group_list, as_index=False).sem()

def filter_min_max_df(df:pd.DataFrame, figure_dict:dict):
    new_df = df.copy()
    if figure_dict.get("x_min") != None:
        new_df = new_df[new_df[figure_dict['x']] >= figure_dict.get("x_min")]
    if figure_dict.get("x_max") != None:
        new_df = new_df[new_df[figure_dict['x']] <= figure_dict.get("x_max")]
    if figure_dict.get("y_min") != None:
        new_df = new_df[new_df[figure_dict['y']] >= figure_dict.get("y_min")]
    if figure_dict.get("y_max") != None:
        new_df = new_df[new_df[figure_dict['y']] <= figure_dict.get("y_max")]
    return new_df


