from PyQt5 import QtWidgets,QtCore,QtGui
from zipfile import ZipFile
from pathlib import Path
import shutil
import os
from pathvalidate import sanitize_filepath
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *

from GraphKey.app.modules.gui.window_elements.project import Project
from GraphKey.app.modules import path_globals

'''
This class is loaded in upon starting the program. From here,
the user can open an existing project, create a new project or
view their templates.
'''
class WelcomePage(QWidget):
    """
    The welcome page class contains introductory dialogue for starting or opening a project
    """
    RECENT_PROJECTS, DATE_MODIFIED = range(2)

    new_project = QtCore.pyqtSignal()
    open_project = QtCore.pyqtSignal(int)
    view_templates = QtCore.pyqtSignal()
    imported_project = QtCore.pyqtSignal(str)
    project_single_clicked = QtCore.pyqtSignal()

    def __init__(self):
        """
        Initialize the UI
        """
        super().__init__()
        self.initUI()

    def initUI(self):
        """
        Creates the welcome/project window as and places intital UI elements inside of it
        """
        WelcomePage = self

        #Creates a vertical box layout
        vboxlayout = QVBoxLayout()
        #And a horizontal box layout
        hboxlayout = QHBoxLayout()

        # Title
        title = QLabel("Microbiology Lab Information Management and Visualization System", self)
        title.setFont(QFont('Arial', 16))

        # Project list
        self.projectList = QTreeView(self)
        self.projectList.setEditTriggers(QAbstractItemView.NoEditTriggers)
        #Adds the project list to the horizontal box layout
        hboxlayout.addWidget(self.projectList)
        self.model = self.createModel(self)
        self.projectList.setModel(self.model)
        self.projectList.doubleClicked.connect(self.projectSelected)
        self.projectList.clicked.connect(self.projectSingleClicked)


        rightmenulayout = QVBoxLayout()
        rightmenugroup = QGroupBox()
        rightmenugroup.setLayout(rightmenulayout)

        # Create project
        createProject = QPushButton("New Project", self)
        createProject.setFixedWidth(160)
        createProject.clicked.connect(self.createProject)
        rightmenulayout.addWidget(createProject)

        #Open Project
        openProject = QPushButton("Open Project", self)
        openProject.setFixedWidth(160)
        openProject.clicked.connect(self.openProject)
        rightmenulayout.addWidget(openProject)

        #View and edit graph templates
        templates = QPushButton("View Templates", self)
        templates.setFixedWidth(160)
        templates.clicked.connect(self.templates)
        rightmenulayout.addWidget(templates)

        hboxlayout.addWidget(rightmenugroup)

        vboxlayout.addWidget(title)

        hlayoutgroupbox = QGroupBox()
        hlayoutgroupbox.setLayout(hboxlayout)

        vboxlayout.addWidget(hlayoutgroupbox)
    
        self.setLayout(vboxlayout)
        QtCore.QMetaObject.connectSlotsByName(WelcomePage)

    def projectSelected(self, selected):
        """On a project select, lets main window know what project is selected (main window is in charge of switching to graph creation view)
           :param: selected - The selected project.
        """
        self.open_project.emit(selected.row())

    def createModel(self, parent):
        """Creates the model for the QTreeView.
           :param: parent - The parent object for this model.
        """
        model = QStandardItemModel(0, 2, parent)
        model.setHeaderData(self.RECENT_PROJECTS, Qt.Horizontal, "Recent Projects")
        model.setHeaderData(self.DATE_MODIFIED, Qt.Horizontal, "Date Modified")
        return model

    def addProject(self, project:Project):
        """
        Adds a created project to the list
        """
        addedProject = []
        addedProject.append(QStandardItem(project.name))
        addedProject.append(QStandardItem(project.dateModifiedFormatted()))
        self.model.appendRow(addedProject)
        #self.model.setData(self.model.index(0, self.RECENT_PROJECTS), project.name)
        #self.model.setData(self.model.index(0, self.DATE_MODIFIED), project.dateModifiedFormatted())

    def createProject(self):
        """
        Lets the main window know to create a new project. 
        """
        self.new_project.emit()
    
    def openProject(self):
        """
        Opens a project selected 
        """
        print(self.model.item(self.projectList.currentIndex().row(), 0).text())
        self.projectSelected(self.projectList.currentIndex())
        
    def projectSingleClicked(self):
        self.project_single_clicked.emit()

    def templates(self):
        self.view_templates.emit()

    
    def importProject(self):
        """Allows users to select a .zip file, which is copied into the saved data folder, and extracted"""
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        home_dir = str(Path.home())
        fileName = QFileDialog.getOpenFileName(self,"QFileDialog.getOpenFileName()", home_dir,"Zip Files (*.zip);;All Files (*)")
        if fileName and fileName[0]:
            title = fileName[0].split("/")
            file_title = title[-1] 
            project_title_split = file_title.split(".")  
            project_title = project_title_split[0]  
            project_path_str = path_globals.projects_folder_path_str + sanitize_filepath(project_title)
            if not os.path.exists(project_path_str): 
                shutil.copy(fileName[0], path_globals.projects_folder_path_str) 
                new_file_path = path_globals.projects_folder_path_str + file_title 
                with ZipFile(new_file_path, 'r') as zip:                
                    zip.extractall(path_globals.projects_folder_path_str)
                os.remove(path_globals.projects_folder_path_str + project_title + ".zip")
                self.imported_project.emit(project_title)
            else: 
                self.show_alert("Project with imported project name already exists. Failed to import Project.")              
            

    def get_all_file_paths(self, directory):
        """Grabs all files within a directory and returns their filepaths in a collection
           :param: directory - The directory to graph file paths from.
           :return: List of file paths in directory.
        """ 
        file_paths = []         
        for root, directories, files in os.walk(directory): 
            for filename in files: 
                filepath = os.path.join(root, filename) 
                file_paths.append(filepath) 
        return file_paths 

    def exportProject(self):
        """Exports the current project into a zip file in the current working directory"""
        project_name = self.model.item(self.projectList.currentIndex().row(), 0).text()
        os.chdir(path_globals.projects_folder_path)     
        file_paths = self.get_all_file_paths(project_name) 
        with ZipFile(project_name + '.zip','w') as zip: 
            for file in file_paths: 
                zip.write(file)
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        home_dir = str(Path.home())
        fileName = str(QFileDialog.getExistingDirectory(self, "Select Directory to Export Project"))        
        if fileName:
            shutil.move(os.getcwd() + "\\" + project_name + ".zip", fileName)
        os.chdir(path_globals.cwd_path_str)
        print(os.getcwd())

    def show_alert(self, text):
        """Shows a basic alert message.
           :param: text - The message to show.
        """
        alert = QMessageBox()
        alert.setWindowTitle("Alert")
        alert.setText(text)
        alert.setStandardButtons(QMessageBox.Ok)
        alert.exec_()

