from PyQt5.QtWidgets import *
from GraphKey.app.modules.gui.graph_creation_window import *


class ScrollableCheckList(QScrollArea):
    """QScrollArea that holds a vertical list of checkboxes"""

    checkboxes_list = []
    # dictionary that maps a parent column to an array containing a list of its unique row values
    group_filter_dict = dict()
    # dictionary that maps a parent column to an array containing a list of the checkboxes associated with it
    filter_parent_dict = dict()

    def __init__(self, parent, title, main_window):
        """Creates the ScrollableCheckList object
           :param: parent - The parent QWidget for this object.
           :param: title - A str that well be used to set the text of a QLabel at the top of this object.
           :param: main_window - The GraphCreationWindow that this is a member of.
        """
        super().__init__(parent)
        self.title = title
        self.widget = QWidget()
        self.vbox = QVBoxLayout()
        self.widget.setLayout(self.vbox)
        self.setWidgetResizable(True)
        self.setWidget(self.widget)
        self.vbox.addWidget(QLabel(title))
        self.main_window = main_window
        self.vbox.setAlignment(Qt.AlignTop)

    def setGeometry(self, int, int_, int_1, int_2):
        """Sets the geometry of this object
           :param: int - The x coordinate position.
           :param: int_ - The y coordinate position.
           :param: int_1 - The width.
           :param: int_2 - The height.
        """
        super().setGeometry(int, int_, int_1, int_2)

    def addCheckBox(self, checkbox: QCheckBox):
        """Adds the checkbox "checkbox" to the vertical box layout
           :param: checkbox - The checkbox to add to the layout.
        """
        self.vbox.addWidget(checkbox)

    def addCheckBoxes(self, checkbox_title_list):
        """Adds multiple checkboxes to this widget using the strings in checkbox_title_list
           :param: checkbox_title_list - The list of titles for each checkbox to be added.
        """
        for checkbox_title in checkbox_title_list:
            checkBox = QCheckBox(checkbox_title)
            self.addCheckBox(checkBox)
            self.checkboxes_list.append(checkBox)

    def addGroupSelectCheckBoxes(self, checkbox_title_list):
        """Adds multiple checkboxes to this widget using the strings in checkbox_title_list. Has state changed signal connected to method that adds this columns associated unique group values to group filtering list.
           :param: checkbox_title_list - The list of titles for each checkbox to be added.
        """
        for checkbox_title in checkbox_title_list:
            checkBox = QCheckBox(checkbox_title)
            checkBox.stateChanged.connect(self.main_window.group_checkbox_state_changed)
            self.addCheckBox(checkBox)
            self.checkboxes_list.append(checkBox)

    def addGroupFilterCheckboxes(self, parent_column, checkbox_title_list):
        """Adds multiple checkboxes to this widget using the strings in checkbox_title_list.
           :param: parent_column - The column that the filter entries in the checkbox_title_list are part of.
           :param: checkbox_title_list - The list of filter entries to be added that correspond with the parent_column.
        """
        # create dictionary entry for this column parent_column for re-adding and removing data
        self.group_filter_dict[parent_column] = []
        for checkbox_title in checkbox_title_list:
            # add entries to dictionary
            self.group_filter_dict[parent_column].append(checkbox_title)
        # clear the layout
        self.clearCheckboxesBeforeReadd()
        # readd everything
        for key in self.group_filter_dict:
            add_list = self.group_filter_dict[key]
            self.filter_parent_dict[key] = []
            for list_item in add_list:
                checkBox = QCheckBox(list_item)
                self.addCheckBox(checkBox)
                self.checkboxes_list.append(checkBox)
                self.filter_parent_dict[key].append(checkBox)
        # self.vbox.insertStretch(-1, 1)

    def readdGroupFilterCheckboxes(self):
        """Method for readding checkboxes after an entry has been remove from the group_filer_dict"""
        # clear the layout
        self.clearCheckboxesBeforeReadd()
        # readd everything
        for key in self.group_filter_dict:
            add_list = self.group_filter_dict[key]
            # create entry in filter_parent_dict for this key
            self.filter_parent_dict[key] = []
            for list_item in add_list:
                checkBox = QCheckBox(list_item)
                self.addCheckBox(checkBox)
                self.checkboxes_list.append(checkBox)
        # self.vbox.insertStretch(-1, 1)

    def clearCheckboxes(self):
        """Removes all of the checkboxes from the vertical box layout and clears the checkboxes lists. Clears the dictionary for keeping track of group filtering"""
        self.checkboxes_list = []
        self.group_filter_dict = dict()
        self.filter_parent_dict = dict()
        while self.vbox.count():
            child = self.vbox.takeAt(0)
            if child.widget():
                child.widget().deleteLater()
        # re-add the title
        self.vbox.addWidget(QLabel(self.title))

    def clearCheckboxesBeforeReadd(self):
        """Removes all of the checkboxes from the vertical box layout and clears the checkboxes lists. Doesn't clear dictionary for keeping track of group filtering"""
        self.checkboxes_list = []
        self.filter_parent_dict = dict()
        while self.vbox.count():
            child = self.vbox.takeAt(0)
            if child.widget():
                child.widget().deleteLater()
        # re-add the title
        self.vbox.addWidget(QLabel(self.title))

    def getListOfCheckedCheckBoxes(self):
        """Returns a list of the text for all checkboxes that are checked"""
        checked_list = []
        for checkBox in self.checkboxes_list:
            if checkBox.isChecked():
                checked_list.append(checkBox.text())
        return checked_list

    def getListOfFilterCheckedCheckBoxesDict(self):
        """Return a dictionary that maps a parent column to a list of the unique filter values that have their associated checkboxes checked"""
        return_dict = dict()
        # get the list of checkboxes associated with each parent column
        for key in self.filter_parent_dict:
            check_box_list = self.filter_parent_dict[key]
            return_dict[key] = []
            # append the names of checked checkboxes to an array for that parent column in the dict
            for checkBox in check_box_list:
                if checkBox.isChecked():
                    return_dict[key].append(checkBox.text())
        return return_dict

    def removeCheckBoxesOfGroupFilterColumn(self, column):
        """Removes all of the group filter checkboxes that are associated with a particular column"""
        # clear all checklists and re-add those that weren't associated with parentColumn "column"
        self.clearCheckboxesBeforeReadd()
        # remove dictionary entries for this column
        self.group_filter_dict.pop(column)
        # re-add checkboxes
        self.readdGroupFilterCheckboxes()

    def hideAndUncheckCheckBox(self, checkbox_text):
        """Hides and unchecks a checkbox.
           :param: checkbox_text - The text of the checkbox to be hidden.
        """
        for checkBox in self.checkboxes_list:
            if checkBox.text() == checkbox_text:
                checkBox.setChecked(False)
                checkBox.hide()

    def showCheckBox(self, checkbox_text):
        """Shows a checkbox.
           :param: checkbox_text - The text of the checkbox to be shown.
        """
        for checkBox in self.checkboxes_list:
            if checkBox.text() == checkbox_text:
                checkBox.show()