import abc

import pandas
import plotly.graph_objects as go


class Figure(abc.ABC):
    """
    This is an abstract container class that holds the Plotly figure object. All that gets passed is the pandas
    DataFrame and a config dictionary or JSON object that holds all the information. Plotly then generates the figure
    and is held here.

    This is also where the other graphing figures inherit from (eg. Boxplot, Barchart, Scatter, etc.)
    """
    @abc.abstractmethod
    def __init__(self, df: pandas.DataFrame):
        pass

    @abc.abstractmethod
    def construct_figure(self, config: dict):
        """
        Constructs the figure object to be passed for graphing.
        :param config: A dictionary that contains all the parameters to be passed to Plotly's Figure object
        """
        pass

    @abc.abstractmethod
    def get_figure(self) -> go.Figure:
        """
        Get's the constructed figure object to be displayed
        :return: A Plotly Figure object
        """
        pass

    @staticmethod
    def eliminate_nones(config: dict):
        """
        Iterates through the passed config and removes all unnecessary empty values
        :param config: A dictionary that contains all the parameters to be passed to Plotly's Figure object
        """
        fig_config = config
        for key, value in fig_config:
            if value is None or value == "None":
                del fig_config[key]
