import pandas as pd
import plotly.express as px
from PyQt5.QtWidgets import *
from GraphKey.app.modules.data_import.data_frame_functions import *
from GraphKey.app.modules.graphing.figure.figure_factory import FigureFactory, SupportedFigures

def create_scatter_graph(data_frame, figure_dict:dict):  
    """Creates a scatter graph using a data_frame and a figure_dict which contains values to all needed variables
       :param: data_frame - Data frame with data used to create the figure.
       :param: figure_dict - dict with configuration information for graph.
       :return: Plotly figure object based on passed in parameters.
    """
    calc_image_config(figure_dict)
    plotly_dict = figure_dict.copy()     
    plotly_dict["color"] = ' and '.join(figure_dict["color_grouping_variables"])
    color_grouping_df = create_df_with_new_grouped_column(data_frame, figure_dict["color_grouping_variables"], ' and ')
    plotly_dict["symbol"] = ' and '.join(figure_dict["marker_style_variables"])
    color_and_marker_df = create_df_with_new_grouped_column(color_grouping_df, figure_dict["marker_style_variables"], ' and ')
    f_df = filter_min_max_df(color_and_marker_df,plotly_dict)
    remove_non_plotly_dict_values(plotly_dict)
    figure_object = FigureFactory.factory(f_df, SupportedFigures.Scatter)
    figure_object.construct_figure(plotly_dict)    
    figure = figure_object.get_figure()
    figure.update_layout(legend=dict(xanchor="left", x=1, y=0.5))    
    set_font_settings(figure, figure_dict)   
    return figure

def create_bar_graph(data_frame, figure_dict:dict):
    """Creates a bar graph using a data_frame and a figure_dict which contains values to all needed variables
       :param: data_frame - Data frame with data used to create the figure.
       :param: figure_dict - dict with configuration information for graph.
       :return: Plotly figure object based on passed in parameters.
    """
    sub_group_enable = False
    if len(figure_dict.get("sub_group_list")) != 0:
        figure_dict["color"] = ' and '.join(figure_dict["sub_group_list"])
        figure_dict["barmode"] = "group"
        sub_group_enable = True
    calc_image_config(figure_dict)
    plotly_dict = figure_dict.copy()    
    filtered_df = filter_using_filter_dict(data_frame, figure_dict["filter_dict"])    
    grouped_df = filtered_df.groupby(figure_dict["group_list"] + figure_dict["sub_group_list"], as_index=False).mean()
    grouped_df = create_df_with_new_grouped_column(grouped_df, figure_dict["group_list"], ' and ')
    if sub_group_enable:
        grouped_df = create_df_with_new_grouped_column(grouped_df, figure_dict["sub_group_list"], ' and ')
    if plotly_dict["enable_error_bars"]:
        sem_df = get_sem_df_for_groups_in_column(filtered_df, figure_dict["group_list"] + figure_dict["sub_group_list"])
        grouped_df["graph_key_error"]=sem_df[figure_dict["y"]]
        plotly_dict["error_y"] = "graph_key_error"
    else:
        if plotly_dict.get("error_y") != None:
            plotly_dict.pop("error_y")
    f_df = filter_min_max_df(grouped_df,plotly_dict)
    remove_non_plotly_dict_values(plotly_dict)
    figure_object = FigureFactory.factory(f_df, SupportedFigures.Bar)
    figure_object.construct_figure(plotly_dict)    
    figure = figure_object.get_figure()
    figure.update_layout(legend=dict(xanchor="left", x=1, y=0.5))
    set_font_settings(figure, figure_dict)
    return figure

def create_box_graph(data_frame, figure_dict:dict):
    """Creates a box graph using a data_frame and a figure_dict which contains values to all needed variables     
       :param: data_frame - Data frame with data used to create the figure.
       :param: figure_dict - dict with configuration information for graph.
       :return: Plotly figure object based on passed in parameters.
    """
    calc_image_config(figure_dict)
    plotly_dict = figure_dict.copy()
    plotly_dict["color"] = ' and '.join(figure_dict["color_grouping_variables"])
    color_grouping_df = create_df_with_new_grouped_column(data_frame, figure_dict["color_grouping_variables"], ' and ')
    #filter out groups as specified in filter dictionary
    filtered_df = filter_using_filter_dict(color_grouping_df, figure_dict["filter_dict"])    
    grouped_df = create_df_with_new_grouped_column(filtered_df, figure_dict["group_list"], ' and ')
    f_df = filter_min_max_df(grouped_df,plotly_dict)
    remove_non_plotly_dict_values(plotly_dict)
    figure_object = FigureFactory.factory(f_df, SupportedFigures.Box)
    figure_object.construct_figure(plotly_dict)    
    figure = figure_object.get_figure()
    figure.update_layout(legend=dict(xanchor="left", x=1, y=0.5))
    set_font_settings(figure, figure_dict)
    return figure

def calc_image_config(figure_dict:dict):
    """Calculates the pixel width and height of the image as well as a scaling factor based on the specified DPI.
       :param: figure_dict - dict with configuration information for graph.
    """
    if figure_dict.get("dpi") == None:
        figure_dict["dpi"]=300
    
    if figure_dict.get("width_inches") == None:
        figure_dict["width_inches"]=6

    if figure_dict.get("height_inches") == None:
        figure_dict["height_inches"]=6

    default_dpi = 96

    unscaled_pixel_width=round(figure_dict["width_inches"] * default_dpi)
    unscaled_pixel_height=round(figure_dict["height_inches"] * default_dpi)

    figure_dict["width"]=unscaled_pixel_width
    figure_dict["height"]=unscaled_pixel_height
    figure_dict["scale_factor"]=figure_dict["dpi"]/default_dpi

def set_font_settings(figure, figure_dict):
    """Sets the font settings for the graph
       :param: figure - The figure object.
       :param: figure_dict - The dictionary containing information about the supplied figure object.
    """
    figure_elements = ["Title","X-Axis Title", "Y-Axis Title", "X-Axis Labels", "Y-Axis Labels", "Legend Title", "Legend Elements"]    
    plotly_key_dict = {"Title":"title_font_", "X-Axis Title": "title_font_", "Y-Axis Title": "title_font_", "X-Axis Labels": "tickfont_", "Y-Axis Labels": "tickfont_", "Legend Title":"legend_title_font_", "Legend Elements": "legend_font_"}
    font_settings = ["family", "size"]
    
    for setting in font_settings:
        for element in figure_elements:
            if figure_dict.get(element + " font " + setting) != None:
                update_dict = {plotly_key_dict[element] + setting: figure_dict[element + " font " + setting]}
                if (element == "Title" or element == "Legend Title" or element == "Legend Elements"):                    
                    figure.update_layout(**update_dict)
                elif element == "X-Axis Title" or element == "X-Axis Labels":
                    figure.update_xaxes(**update_dict)
                elif element == "Y-Axis Title" or element == "Y-Axis Labels":
                    figure.update_yaxes(**update_dict)


def remove_non_plotly_dict_values(figure_dict):
    """Removes dictionary entries that have been saved to the graph's pickle file but aren't needed as kwargs for plotly graph generation
       :param: figure_dict - The dictionary that will have non plotly dict values filtered from it.
    """
    if "x_min" in figure_dict:
        figure_dict.pop("x_min")
    if "x_max" in figure_dict:
        figure_dict.pop("x_max")
    if "y_min" in figure_dict:
        figure_dict.pop("y_min")
    if "y_max" in figure_dict:
        figure_dict.pop("y_max")

    non_plotly_entries = ["color_grouping_variables", "marker_style_variables", "sheet_selected", "graph_type", "group_list", "filter_dict", "width_inches", "height_inches", "dpi", "scale_factor", "enable_error_bars", "sub_group_list"]
    for entry in non_plotly_entries:
        if figure_dict.get(entry) != None:
            figure_dict.pop(entry)

    figure_elements = ["Title","X-Axis Title", "Y-Axis Title", "X-Axis Labels", "Y-Axis Labels", "Legend Title", "Legend Elements"]

    for element in figure_elements:
        if figure_dict.get(element + " font family") != None:
            figure_dict.pop(element + " font family")
        if figure_dict.get(element + " font size") != None:
            figure_dict.pop(element + " font size")

def update_scale(figure, figure_dict):
    """Updates the scale of the graph based on x_min/y_min and x_max/y_max values in figure_dict
       :param: figure - The figure object being configured.
       :param: figure_dict - dict containing configuration information for the figure object.
    """
    if figure_dict.get("x_min") != None and figure_dict.get("x_max") != None:
        figure.update_xaxes(range=[figure_dict.get("x_min"),figure_dict.get("x_max")],automargin=True)
    if figure_dict.get("y_min") != None and figure_dict.get("y_max") != None:
        figure.update_yaxes(range=[figure_dict.get("y_min"),figure_dict.get("y_max")],automargin=True)


